from pyetp.resqml_objects.generated import (
    APIGammaRayMeasure,
    APIGammaRayUom,
    APIGravityMeasure,
    APIGravityUom,
    APINeutronMeasure,
    APINeutronUom,
    AbsorbedDoseMeasure,
    AbsorbedDoseUom,
    AbstractActivityParameter,
    AbstractBooleanArray,
    AbstractCRSType,
    AbstractCitedDataObject,
    AbstractColumnLayerGridGeometry,
    AbstractColumnLayerGridRepresentation,
    AbstractContactInterpretationPart,
    AbstractContactRepresentationPart,
    AbstractContextualObject,
    AbstractCoordinateOperationType,
    AbstractCoordinateSystemType,
    AbstractDQ_Element_Type,
    AbstractDQ_PositionalAccuracy_Type,
    AbstractDQ_Result_Type,
    AbstractDataObject,
    AbstractDatumType,
    AbstractDoubleArray,
    AbstractEX_GeographicExtent_Type,
    AbstractFeature,
    AbstractFeatureInterpretation,
    AbstractGMLType,
    AbstractGeneralConversionType,
    AbstractGeneralDerivedCRSType,
    AbstractGeologicFeature,
    AbstractGeometry,
    AbstractGridGeometry,
    AbstractGridRepresentation,
    AbstractIntegerArray,
    AbstractLocal3dCrs,
    AbstractObject,
    AbstractObject_Type,
    AbstractOrganizationInterpretation,
    AbstractParameterKey,
    AbstractParametricLineArray,
    AbstractParametricLineGeometry,
    AbstractParentWindow,
    AbstractPlaneGeometry,
    AbstractPoint3dArray,
    AbstractProjectedCrs,
    AbstractProperty,
    AbstractPropertyKind,
    AbstractPropertyLookup,
    AbstractRepresentation,
    AbstractResqmlDataObject,
    AbstractSeismicCoordinates,
    AbstractSeismicSurveyFeature,
    AbstractStratigraphicOrganizationInterpretation,
    AbstractSurfaceFrameworkRepresentation,
    AbstractSurfaceRepresentation,
    AbstractTechnicalFeature,
    AbstractTimeObjectType,
    AbstractTimePrimitiveType,
    AbstractTruncatedColumnLayerGridRepresentation,
    AbstractValueArray,
    AbstractValuesProperty,
    AbstractVerticalCrs,
    Activation,
    Activity,
    ActivityOfRadioactivityMeasure,
    ActivityOfRadioactivityUom,
    ActivityTemplate,
    AdditionalGridPoints,
    AdditionalGridTopology,
    AggregationType,
    AmountOfSubstanceMeasure,
    AmountOfSubstancePerAmountOfSubstanceMeasure,
    AmountOfSubstancePerAmountOfSubstanceUom,
    AmountOfSubstancePerAreaMeasure,
    AmountOfSubstancePerAreaUom,
    AmountOfSubstancePerTimeMeasure,
    AmountOfSubstancePerTimePerAreaMeasure,
    AmountOfSubstancePerTimePerAreaUom,
    AmountOfSubstancePerTimeUom,
    AmountOfSubstancePerVolumeMeasure,
    AmountOfSubstancePerVolumeUom,
    AmountOfSubstanceUom,
    AnglePerLengthMeasure,
    AnglePerLengthUom,
    AnglePerVolumeMeasure,
    AnglePerVolumeUom,
    AngularAccelerationMeasure,
    AngularAccelerationUom,
    AngularVelocityMeasure,
    AngularVelocityUom,
    AreaMeasure,
    AreaPerAmountOfSubstanceMeasure,
    AreaPerAmountOfSubstanceUom,
    AreaPerAreaMeasure,
    AreaPerAreaUom,
    AreaPerMassMeasure,
    AreaPerMassUom,
    AreaPerTimeMeasure,
    AreaPerTimeUom,
    AreaPerVolumeMeasure,
    AreaPerVolumeUom,
    AreaUom,
    AttenuationPerFrequencyIntervalMeasure,
    AttenuationPerFrequencyIntervalUom,
    AxisOrder2d,
    BinaryContactInterpretationPart,
    BlockedWellboreRepresentation,
    Boolean,
    BooleanArrayFromDiscretePropertyArray,
    BooleanArrayFromIndexArray,
    BooleanConstantArray,
    BooleanHdf5Array,
    Boolean_PropertyType,
    BoundaryFeature,
    BoundaryFeatureInterpretation,
    BoundaryRelation,
    CI_Address,
    CI_Address_PropertyType,
    CI_Address_Type,
    CI_Citation,
    CI_Citation_PropertyType,
    CI_Citation_Type,
    CI_Contact,
    CI_Contact_PropertyType,
    CI_Contact_Type,
    CI_Date,
    CI_DateTypeCode,
    CI_DateTypeCode_PropertyType,
    CI_Date_PropertyType,
    CI_Date_Type,
    CI_OnLineFunctionCode,
    CI_OnLineFunctionCode_PropertyType,
    CI_OnlineResource,
    CI_OnlineResource_PropertyType,
    CI_OnlineResource_Type,
    CI_PresentationFormCode,
    CI_PresentationFormCode_PropertyType,
    CI_ResponsibleParty,
    CI_ResponsibleParty_PropertyType,
    CI_ResponsibleParty_Type,
    CI_RoleCode,
    CI_RoleCode_PropertyType,
    CI_Series,
    CI_Series_PropertyType,
    CI_Series_Type,
    CI_Telephone,
    CI_Telephone_PropertyType,
    CI_Telephone_Type,
    CRSPropertyType,
    CapacitanceMeasure,
    CapacitanceUom,
    CartesianCSPropertyType,
    CartesianCSType,
    CartesianCS_1,
    CategoricalProperty,
    CategoricalPropertySeries,
    CellFluidPhaseUnits,
    CellOverlap,
    CellParentWindow,
    CellShape,
    CellStratigraphicUnits,
    CharacterString,
    CharacterString_PropertyType,
    ChronostratigraphicRank,
    Citation,
    CodeListValue_Type,
    CodeType,
    CodeWithAuthorityType,
    ColumnLayerParentWindow,
    ColumnLayerSplitColumnEdges,
    ColumnLayerSplitCoordinateLines,
    ColumnLayerSubnodeTopology,
    ColumnShape,
    ColumnSubnodePatch,
    CommentProperty,
    CommentPropertySeries,
    ConnectionInterpretations,
    ContactElementReference,
    ContactIdentity,
    ContactMode,
    ContactPatch,
    ContactRelationship,
    ContactRepresentationReference,
    ContactSide,
    ContactVerb,
    ContinuousProperty,
    ContinuousPropertySeries,
    CoordinateSystemAxis,
    CoordinateSystemAxisPropertyType,
    CoordinateSystemAxisType,
    CustomData,
    DQ_EvaluationMethodTypeCode,
    DQ_EvaluationMethodTypeCode_PropertyType,
    DQ_Result_PropertyType,
    DataObjectParameter,
    DataObjectReference,
    DataTransferSpeedMeasure,
    DataTransferSpeedUom,
    Date,
    DateTime,
    DateTime_PropertyType,
    Date_PropertyType,
    Definition,
    DefinitionBaseType,
    DefinitionType,
    DepositionMode,
    DeviationSurveyRepresentation,
    DiffusionCoefficientMeasure,
    DiffusionCoefficientUom,
    DigitalStorageMeasure,
    DigitalStorageUom,
    DimensionlessMeasure,
    DimensionlessUom,
    DipoleMomentMeasure,
    DipoleMomentUom,
    DiscreteProperty,
    DiscretePropertySeries,
    Domain,
    DoseEquivalentMeasure,
    DoseEquivalentUom,
    DoubleConstantArray,
    DoubleHdf5Array,
    DoubleLatticeArray,
    DoubleLookup,
    DoubleTableLookup,
    DynamicViscosityMeasure,
    DynamicViscosityUom,
    EX_Extent,
    EX_Extent_Type,
    EX_GeographicExtent_PropertyType,
    EX_TemporalExtent,
    EX_TemporalExtent_PropertyType,
    EX_TemporalExtent_Type,
    EX_VerticalExtent,
    EX_VerticalExtent_PropertyType,
    EX_VerticalExtent_Type,
    EarthModelInterpretation,
    EdgePatch,
    Edges,
    ElectricChargeMeasure,
    ElectricChargePerAreaMeasure,
    ElectricChargePerAreaUom,
    ElectricChargePerMassMeasure,
    ElectricChargePerMassUom,
    ElectricChargePerVolumeMeasure,
    ElectricChargePerVolumeUom,
    ElectricChargeUom,
    ElectricConductanceMeasure,
    ElectricConductanceUom,
    ElectricConductivityMeasure,
    ElectricConductivityUom,
    ElectricCurrentDensityMeasure,
    ElectricCurrentDensityUom,
    ElectricCurrentMeasure,
    ElectricCurrentUom,
    ElectricFieldStrengthMeasure,
    ElectricFieldStrengthUom,
    ElectricPotentialDifferenceMeasure,
    ElectricPotentialDifferenceUom,
    ElectricResistanceMeasure,
    ElectricResistancePerLengthMeasure,
    ElectricResistancePerLengthUom,
    ElectricResistanceUom,
    ElectricalResistivityMeasure,
    ElectricalResistivityUom,
    ElectromagneticMomentMeasure,
    ElectromagneticMomentUom,
    ElementIdentity,
    ElementIndices,
    EllipsoidPropertyType,
    EllipsoidType,
    Ellipsoid_1,
    EllipsoidalCSPropertyType,
    EllipsoidalCSType,
    EllipsoidalCS_1,
    EnergyLengthPerAreaMeasure,
    EnergyLengthPerAreaUom,
    EnergyLengthPerTimeAreaTemperatureMeasure,
    EnergyLengthPerTimeAreaTemperatureUom,
    EnergyMeasure,
    EnergyPerAreaMeasure,
    EnergyPerAreaUom,
    EnergyPerLengthMeasure,
    EnergyPerLengthUom,
    EnergyPerMassMeasure,
    EnergyPerMassPerTimeMeasure,
    EnergyPerMassPerTimeUom,
    EnergyPerMassUom,
    EnergyPerVolumeMeasure,
    EnergyPerVolumeUom,
    EnergyUom,
    EpcExternalPartReference,
    Facet,
    FaultInterpretation,
    FaultThrow,
    FloatingPointQuantityParameter,
    FluidBoundaryFeature,
    FluidContact,
    FluidMarker,
    ForceAreaMeasure,
    ForceAreaUom,
    ForceLengthPerLengthMeasure,
    ForceLengthPerLengthUom,
    ForceMeasure,
    ForcePerForceMeasure,
    ForcePerForceUom,
    ForcePerLengthMeasure,
    ForcePerLengthUom,
    ForcePerVolumeMeasure,
    ForcePerVolumeUom,
    ForceUom,
    FrequencyIntervalMeasure,
    FrequencyIntervalUom,
    FrequencyMeasure,
    FrequencyUom,
    FrontierFeature,
    GeneralConversionPropertyType,
    GenericFeatureInterpretation,
    GeneticBoundaryFeature,
    GeneticBoundaryKind,
    Geobody3dShape,
    GeobodyBoundaryInterpretation,
    GeobodyFeature,
    GeobodyInterpretation,
    GeodeticCRS,
    GeodeticCRSPropertyType,
    GeodeticCRSType,
    GeodeticDatumPropertyType,
    GeodeticDatumType,
    GeodeticDatum_1,
    GeologicBoundaryKind,
    GeologicUnitComposition,
    GeologicUnitFeature,
    GeologicUnitInterpretation,
    GeologicUnitInterpretationIndex,
    GeologicUnitMaterialImplacement,
    GlobalChronostratigraphicColumn,
    GmlProjectedCrsDefinition,
    GmlVerticalCrsDefinition,
    GpGridColumnLayerGrid,
    GpGridIjkGridPatch,
    GpGridRepresentation,
    GpGridUnstructuredColumnLayerGridPatch,
    GpGridUnstructuredGridPatch,
    Grid2dPatch,
    Grid2dRepresentation,
    Grid2dSetRepresentation,
    GridConnectionSetRepresentation,
    GridGeometryAttachment,
    Hdf5Dataset,
    HeatCapacityMeasure,
    HeatCapacityUom,
    HeatFlowRateMeasure,
    HeatFlowRateUom,
    HeatTransferCoefficientMeasure,
    HeatTransferCoefficientUom,
    HorizonInterpretation,
    HorizonInterpretationIndex,
    HorizontalPlaneGeometry,
    IdentifiedObjectType,
    IdentityKind,
    IjGaps,
    IjSplitColumnEdges,
    IjkGridGeometry,
    IjkGridRepresentation,
    IjkParentWindow,
    IlluminanceMeasure,
    IlluminanceUom,
    IndexableElements,
    InductanceMeasure,
    InductanceUom,
    IntegerArrayFromBooleanMaskArray,
    IntegerConstantArray,
    IntegerHdf5Array,
    IntegerLatticeArray,
    IntegerQuantityParameter,
    IntegerRangeArray,
    IntervalGridCells,
    IntervalStratigraphicUnits,
    Intervals,
    IsothermalCompressibilityMeasure,
    IsothermalCompressibilityUom,
    KDirection,
    KGaps,
    KinematicViscosityMeasure,
    KinematicViscosityUom,
    LengthMeasure,
    LengthPerLengthMeasure,
    LengthPerLengthUom,
    LengthPerMassMeasure,
    LengthPerMassUom,
    LengthPerPressureMeasure,
    LengthPerPressureUom,
    LengthPerTemperatureMeasure,
    LengthPerTemperatureUom,
    LengthPerTimeMeasure,
    LengthPerTimeUom,
    LengthPerVolumeMeasure,
    LengthPerVolumeUom,
    LengthUom,
    LightExposureMeasure,
    LightExposureUom,
    LineRole,
    LinearAccelerationMeasure,
    LinearAccelerationUom,
    LinearThermalExpansionMeasure,
    LinearThermalExpansionUom,
    LocalDepth3dCrs,
    LocalGridSet,
    LocalPropertyKind,
    LocalTime3dCrs,
    LogarithmicPowerRatioMeasure,
    LogarithmicPowerRatioPerLengthMeasure,
    LogarithmicPowerRatioPerLengthUom,
    LogarithmicPowerRatioUom,
    LuminanceMeasure,
    LuminanceUom,
    LuminousEfficacyMeasure,
    LuminousEfficacyUom,
    LuminousFluxMeasure,
    LuminousFluxUom,
    LuminousIntensityMeasure,
    LuminousIntensityUom,
    MD_Identifier,
    MD_Identifier_PropertyType,
    MD_Identifier_Type,
    MagneticDipoleMomentMeasure,
    MagneticDipoleMomentUom,
    MagneticFieldStrengthMeasure,
    MagneticFieldStrengthUom,
    MagneticFluxDensityMeasure,
    MagneticFluxDensityPerLengthMeasure,
    MagneticFluxDensityPerLengthUom,
    MagneticFluxDensityUom,
    MagneticFluxMeasure,
    MagneticFluxUom,
    MagneticPermeabilityMeasure,
    MagneticPermeabilityUom,
    MagneticVectorPotentialMeasure,
    MagneticVectorPotentialUom,
    MassLengthMeasure,
    MassLengthUom,
    MassMeasure,
    MassPerAreaMeasure,
    MassPerAreaUom,
    MassPerEnergyMeasure,
    MassPerEnergyUom,
    MassPerLengthMeasure,
    MassPerLengthUom,
    MassPerMassMeasure,
    MassPerMassUom,
    MassPerTimeMeasure,
    MassPerTimePerAreaMeasure,
    MassPerTimePerAreaUom,
    MassPerTimePerLengthMeasure,
    MassPerTimePerLengthUom,
    MassPerTimeUom,
    MassPerVolumeMeasure,
    MassPerVolumePerLengthMeasure,
    MassPerVolumePerLengthUom,
    MassPerVolumeUom,
    MassUom,
    MdDatum,
    MdDomain,
    MdReference,
    MobilityMeasure,
    MobilityUom,
    MolarEnergyMeasure,
    MolarEnergyUom,
    MolarHeatCapacityMeasure,
    MolarHeatCapacityUom,
    MolarVolumeMeasure,
    MolarVolumeUom,
    MolecularWeightMeasure,
    MolecularWeightUom,
    MomentOfForceMeasure,
    MomentOfForceUom,
    MomentOfInertiaMeasure,
    MomentOfInertiaUom,
    MomentumMeasure,
    MomentumUom,
    MultipleContactInterpretationPart,
    NameValuePair,
    NilReasonEnumeration_value,
    NodePatch,
    NodesPerCell,
    NonSealedContactRepresentationPart,
    NonSealedSurfaceFrameworkRepresentation,
    NormalizedPowerMeasure,
    NormalizedPowerUom,
    ObjectAlias,
    ObjectParameterKey,
    OrderingCriteria,
    OrganizationFeature,
    OrganizationKind,
    OrientedMacroFace,
    OverlapVolume,
    ParameterKind,
    ParameterTemplate,
    ParametricLineArray,
    ParametricLineFromRepresentationGeometry,
    ParametricLineFromRepresentationLatticeArray,
    ParametricLineGeometry,
    ParametricLineIntersections,
    Patch,
    Patch1d,
    PatchBoundaries,
    PatchOfGeometry,
    PatchOfPoints,
    PatchOfValues,
    PermeabilityLengthMeasure,
    PermeabilityLengthUom,
    PermeabilityRockMeasure,
    PermeabilityRockUom,
    PermittivityMeasure,
    PermittivityUom,
    Phase,
    PillarShape,
    PlaneAngleMeasure,
    PlaneAngleUom,
    PlaneSetRepresentation,
    Point2dHdf5Array,
    Point3d,
    Point3dFromRepresentationLatticeArray,
    Point3dHdf5Array,
    Point3dLatticeArray,
    Point3dOffset,
    Point3dParametricArray,
    Point3dZValueArray,
    PointGeometry,
    PointSetRepresentation,
    PointsProperty,
    PolylineRepresentation,
    PolylineSetPatch,
    PolylineSetRepresentation,
    PotentialDifferencePerPowerDropMeasure,
    PotentialDifferencePerPowerDropUom,
    PowerMeasure,
    PowerPerAreaMeasure,
    PowerPerAreaUom,
    PowerPerPowerMeasure,
    PowerPerPowerUom,
    PowerPerVolumeMeasure,
    PowerPerVolumeUom,
    PowerUom,
    PressureMeasure,
    PressurePerTimeMeasure,
    PressurePerTimeUom,
    PressurePerVolumeMeasure,
    PressurePerVolumeUom,
    PressureSquaredMeasure,
    PressureSquaredPerForceTimePerAreaMeasure,
    PressureSquaredPerForceTimePerAreaUom,
    PressureSquaredUom,
    PressureTimePerVolumeMeasure,
    PressureTimePerVolumeUom,
    PressureUom,
    PrimeMeridianPropertyType,
    PrimeMeridianType,
    PrimeMeridian_1,
    ProjectedCRS,
    ProjectedCRSType,
    ProjectedCrsEpsgCode,
    ProjectedCrs_1,
    ProjectedUnknownCrs,
    PropertyKind,
    PropertyKindFacet,
    PropertySet,
    PropertyValuesPatch,
    QuantityOfLightMeasure,
    QuantityOfLightUom,
    RadianceMeasure,
    RadianceUom,
    RadiantIntensityMeasure,
    RadiantIntensityUom,
    Real,
    Real_PropertyType,
    ReciprocalAreaMeasure,
    ReciprocalAreaUom,
    ReciprocalElectricPotentialDifferenceMeasure,
    ReciprocalElectricPotentialDifferenceUom,
    ReciprocalForceMeasure,
    ReciprocalForceUom,
    ReciprocalLengthMeasure,
    ReciprocalLengthUom,
    ReciprocalMassMeasure,
    ReciprocalMassTimeMeasure,
    ReciprocalMassTimeUom,
    ReciprocalMassUom,
    ReciprocalPressureMeasure,
    ReciprocalPressureUom,
    ReciprocalTimeMeasure,
    ReciprocalTimeUom,
    ReciprocalVolumeMeasure,
    ReciprocalVolumeUom,
    RedefinedGeometryRepresentation,
    ReferenceType,
    Regrid,
    RelatedTimeType,
    RelatedTimeType_relativePosition,
    ReluctanceMeasure,
    ReluctanceUom,
    RepresentationIdentity,
    RepresentationIdentitySet,
    RepresentationSetRepresentation,
    ResqmlJaggedArray,
    ResqmlPropertyKind,
    ResqmlUom,
    RockFluidOrganizationInterpretation,
    RockFluidUnitFeature,
    RockFluidUnitInterpretation,
    RockFluidUnitInterpretationIndex,
    SC_CRS_PropertyType,
    SealedContactRepresentationPart,
    SealedSurfaceFrameworkRepresentation,
    SealedVolumeFrameworkRepresentation,
    SecondDefiningParameter_1,
    SecondMomentOfAreaMeasure,
    SecondMomentOfAreaUom,
    Seismic2dCoordinates,
    Seismic3dCoordinates,
    SeismicLatticeFeature,
    SeismicLatticeSetFeature,
    SeismicLineFeature,
    SeismicLineSetFeature,
    SequenceStratigraphySurface,
    SignalingEventPerTimeMeasure,
    SignalingEventPerTimeUom,
    SolidAngleMeasure,
    SolidAngleUom,
    SpecificHeatCapacityMeasure,
    SpecificHeatCapacityUom,
    SphericalCSPropertyType,
    SphericalCSType,
    SphericalCS_1,
    SplitEdges,
    SplitFaces,
    SplitNodePatch,
    StandardPropertyKind,
    StratigraphicColumn,
    StratigraphicColumnRankInterpretation,
    StratigraphicOccurrenceInterpretation,
    StratigraphicUnitFeature,
    StratigraphicUnitInterpretation,
    StratigraphicUnitInterpretationIndex,
    StreamlineFlux,
    StreamlinePolylineSetPatch,
    StreamlineWellbores,
    StreamlinesFeature,
    StreamlinesRepresentation,
    StringHdf5Array,
    StringLookup,
    StringOrRefType,
    StringParameter,
    StringTableLookup,
    StructuralOrganizationInterpretation,
    SubRepresentation,
    SubRepresentationPatch,
    SubnodeNodeObject,
    SubnodePatch,
    SubnodeTopology,
    SurfaceRole,
    TM_Primitive_PropertyType,
    TectonicBoundaryFeature,
    TectonicBoundaryKind,
    TemperatureIntervalMeasure,
    TemperatureIntervalPerLengthMeasure,
    TemperatureIntervalPerLengthUom,
    TemperatureIntervalPerPressureMeasure,
    TemperatureIntervalPerPressureUom,
    TemperatureIntervalPerTimeMeasure,
    TemperatureIntervalPerTimeUom,
    TemperatureIntervalUom,
    ThermalConductanceMeasure,
    ThermalConductanceUom,
    ThermalConductivityMeasure,
    ThermalConductivityUom,
    ThermalDiffusivityMeasure,
    ThermalDiffusivityUom,
    ThermalInsulanceMeasure,
    ThermalInsulanceUom,
    ThermalResistanceMeasure,
    ThermalResistanceUom,
    ThermodynamicTemperatureMeasure,
    ThermodynamicTemperatureUom,
    ThreePoint3d,
    ThrowKind,
    TiltedPlaneGeometry,
    TimeIndex,
    TimeIndexParameter,
    TimeIndexParameterKey,
    TimeIndices,
    TimeInterval,
    TimeMeasure,
    TimePerLengthMeasure,
    TimePerLengthUom,
    TimePerMassMeasure,
    TimePerMassUom,
    TimePerTimeMeasure,
    TimePerTimeUom,
    TimePerVolumeMeasure,
    TimePerVolumeUom,
    TimePrimitivePropertyType,
    TimeSeries,
    TimeSeriesParentage,
    TimeSetKind,
    TimeUom,
    Timestamp,
    TrianglePatch,
    TriangulatedSetRepresentation,
    TruncatedIjkGridRepresentation,
    TruncatedUnstructuredColumnLayerGridRepresentation,
    TruncationCellPatch,
    URL,
    URL_PropertyType,
    UniformSubnodePatch,
    UnstructuredColumnEdges,
    UnstructuredColumnLayerGridGeometry,
    UnstructuredColumnLayerGridRepresentation,
    UnstructuredGridGeometry,
    UnstructuredGridHingeNodeFaces,
    UnstructuredGridRepresentation,
    UnstructuredSubnodeTopology,
    VariableSubnodePatch,
    VerticalCRS,
    VerticalCRSType,
    VerticalCSPropertyType,
    VerticalCSType,
    VerticalCS_1,
    VerticalCrsEpsgCode,
    VerticalCrs_1,
    VerticalDatumPropertyType,
    VerticalDatumType,
    VerticalDatum_1,
    VerticalDirection,
    VerticalUnknownCrs,
    VolumeFlowRatePerVolumeFlowRateMeasure,
    VolumeFlowRatePerVolumeFlowRateUom,
    VolumeMeasure,
    VolumePerAreaMeasure,
    VolumePerAreaUom,
    VolumePerLengthMeasure,
    VolumePerLengthUom,
    VolumePerMassMeasure,
    VolumePerMassUom,
    VolumePerPressureMeasure,
    VolumePerPressureUom,
    VolumePerRotationMeasure,
    VolumePerRotationUom,
    VolumePerTimeLengthMeasure,
    VolumePerTimeLengthUom,
    VolumePerTimeMeasure,
    VolumePerTimePerAreaMeasure,
    VolumePerTimePerAreaUom,
    VolumePerTimePerLengthMeasure,
    VolumePerTimePerLengthUom,
    VolumePerTimePerPressureLengthMeasure,
    VolumePerTimePerPressureLengthUom,
    VolumePerTimePerPressureMeasure,
    VolumePerTimePerPressureUom,
    VolumePerTimePerTimeMeasure,
    VolumePerTimePerTimeUom,
    VolumePerTimePerVolumeMeasure,
    VolumePerTimePerVolumeUom,
    VolumePerTimeUom,
    VolumePerVolumeMeasure,
    VolumePerVolumeUom,
    VolumeRegion,
    VolumeShell,
    VolumeUom,
    VolumetricHeatTransferCoefficientMeasure,
    VolumetricHeatTransferCoefficientUom,
    VolumetricThermalExpansionMeasure,
    VolumetricThermalExpansionUom,
    WellboreFeature,
    WellboreFrameRepresentation,
    WellboreInterpretation,
    WellboreMarker,
    WellboreMarkerFrameRepresentation,
    WellboreTrajectoryParentIntersection,
    WellboreTrajectoryRepresentation,
    WitsmlWellboreReference,
    actuate_value,
    anchorDefinition,
    axis,
    axisAbbrev,
    axisDirection,
    baseGeodeticCRS,
    cartesianCS_2,
    conversion,
    coordinateOperationAccuracy,
    description,
    descriptionReference,
    domainOfValidity,
    ellipsoid_2,
    ellipsoidalCS_2,
    geodeticDatum_2,
    greenwichLongitude,
    identifier,
    maximumValue,
    minimumValue,
    name,
    obj_Activity,
    obj_ActivityTemplate,
    obj_BlockedWellboreRepresentation,
    obj_BoundaryFeature,
    obj_BoundaryFeatureInterpretation,
    obj_CategoricalProperty,
    obj_CategoricalPropertySeries,
    obj_CommentProperty,
    obj_CommentPropertySeries,
    obj_ContinuousProperty,
    obj_ContinuousPropertySeries,
    obj_DeviationSurveyRepresentation,
    obj_DiscreteProperty,
    obj_DiscretePropertySeries,
    obj_DoubleTableLookup,
    obj_EarthModelInterpretation,
    obj_EpcExternalPartReference,
    obj_FaultInterpretation,
    obj_FluidBoundaryFeature,
    obj_FrontierFeature,
    obj_GenericFeatureInterpretation,
    obj_GeneticBoundaryFeature,
    obj_GeobodyBoundaryInterpretation,
    obj_GeobodyFeature,
    obj_GeobodyInterpretation,
    obj_GeologicUnitFeature,
    obj_GeologicUnitInterpretation,
    obj_GlobalChronostratigraphicColumn,
    obj_GpGridRepresentation,
    obj_Grid2dRepresentation,
    obj_Grid2dSetRepresentation,
    obj_GridConnectionSetRepresentation,
    obj_HorizonInterpretation,
    obj_IjkGridRepresentation,
    obj_LocalDepth3dCrs,
    obj_LocalGridSet,
    obj_LocalTime3dCrs,
    obj_MdDatum,
    obj_NonSealedSurfaceFrameworkRepresentation,
    obj_OrganizationFeature,
    obj_PlaneSetRepresentation,
    obj_PointSetRepresentation,
    obj_PointsProperty,
    obj_PolylineRepresentation,
    obj_PolylineSetRepresentation,
    obj_PropertyKind,
    obj_PropertySet,
    obj_RedefinedGeometryRepresentation,
    obj_RepresentationIdentitySet,
    obj_RepresentationSetRepresentation,
    obj_RockFluidOrganizationInterpretation,
    obj_RockFluidUnitFeature,
    obj_RockFluidUnitInterpretation,
    obj_SealedSurfaceFrameworkRepresentation,
    obj_SealedVolumeFrameworkRepresentation,
    obj_SeismicLatticeFeature,
    obj_SeismicLineFeature,
    obj_SeismicLineSetFeature,
    obj_StratigraphicColumn,
    obj_StratigraphicColumnRankInterpretation,
    obj_StratigraphicOccurrenceInterpretation,
    obj_StratigraphicUnitFeature,
    obj_StratigraphicUnitInterpretation,
    obj_StreamlinesFeature,
    obj_StreamlinesRepresentation,
    obj_StringTableLookup,
    obj_StructuralOrganizationInterpretation,
    obj_SubRepresentation,
    obj_TectonicBoundaryFeature,
    obj_TimeSeries,
    obj_TriangulatedSetRepresentation,
    obj_TruncatedIjkGridRepresentation,
    obj_TruncatedUnstructuredColumnLayerGridRepresentation,
    obj_UnstructuredColumnLayerGridRepresentation,
    obj_UnstructuredGridRepresentation,
    obj_WellboreFeature,
    obj_WellboreFrameRepresentation,
    obj_WellboreInterpretation,
    obj_WellboreMarkerFrameRepresentation,
    obj_WellboreTrajectoryRepresentation,
    operationVersion,
    primeMeridian_2,
    rangeMeaning,
    realizationEpoch,
    remarks,
    scope,
    secondDefiningParameter_2,
    semiMajorAxis,
    show_value,
    sourceCRS,
    sphericalCS_2,
    targetCRS,
    verticalCS_2,
    verticalDatum_2,
)

__all__ = [
    "APIGammaRayMeasure",
    "APIGammaRayUom",
    "APIGravityMeasure",
    "APIGravityUom",
    "APINeutronMeasure",
    "APINeutronUom",
    "AbsorbedDoseMeasure",
    "AbsorbedDoseUom",
    "AbstractActivityParameter",
    "AbstractBooleanArray",
    "AbstractCRSType",
    "AbstractCitedDataObject",
    "AbstractColumnLayerGridGeometry",
    "AbstractColumnLayerGridRepresentation",
    "AbstractContactInterpretationPart",
    "AbstractContactRepresentationPart",
    "AbstractContextualObject",
    "AbstractCoordinateOperationType",
    "AbstractCoordinateSystemType",
    "AbstractDQ_Element_Type",
    "AbstractDQ_PositionalAccuracy_Type",
    "AbstractDQ_Result_Type",
    "AbstractDataObject",
    "AbstractDatumType",
    "AbstractDoubleArray",
    "AbstractEX_GeographicExtent_Type",
    "AbstractFeature",
    "AbstractFeatureInterpretation",
    "AbstractGMLType",
    "AbstractGeneralConversionType",
    "AbstractGeneralDerivedCRSType",
    "AbstractGeologicFeature",
    "AbstractGeometry",
    "AbstractGridGeometry",
    "AbstractGridRepresentation",
    "AbstractIntegerArray",
    "AbstractLocal3dCrs",
    "AbstractObject",
    "AbstractObject_Type",
    "AbstractOrganizationInterpretation",
    "AbstractParameterKey",
    "AbstractParametricLineArray",
    "AbstractParametricLineGeometry",
    "AbstractParentWindow",
    "AbstractPlaneGeometry",
    "AbstractPoint3dArray",
    "AbstractProjectedCrs",
    "AbstractProperty",
    "AbstractPropertyKind",
    "AbstractPropertyLookup",
    "AbstractRepresentation",
    "AbstractResqmlDataObject",
    "AbstractSeismicCoordinates",
    "AbstractSeismicSurveyFeature",
    "AbstractStratigraphicOrganizationInterpretation",
    "AbstractSurfaceFrameworkRepresentation",
    "AbstractSurfaceRepresentation",
    "AbstractTechnicalFeature",
    "AbstractTimeObjectType",
    "AbstractTimePrimitiveType",
    "AbstractTruncatedColumnLayerGridRepresentation",
    "AbstractValueArray",
    "AbstractValuesProperty",
    "AbstractVerticalCrs",
    "Activation",
    "Activity",
    "ActivityOfRadioactivityMeasure",
    "ActivityOfRadioactivityUom",
    "ActivityTemplate",
    "AdditionalGridPoints",
    "AdditionalGridTopology",
    "AggregationType",
    "AmountOfSubstanceMeasure",
    "AmountOfSubstancePerAmountOfSubstanceMeasure",
    "AmountOfSubstancePerAmountOfSubstanceUom",
    "AmountOfSubstancePerAreaMeasure",
    "AmountOfSubstancePerAreaUom",
    "AmountOfSubstancePerTimeMeasure",
    "AmountOfSubstancePerTimePerAreaMeasure",
    "AmountOfSubstancePerTimePerAreaUom",
    "AmountOfSubstancePerTimeUom",
    "AmountOfSubstancePerVolumeMeasure",
    "AmountOfSubstancePerVolumeUom",
    "AmountOfSubstanceUom",
    "AnglePerLengthMeasure",
    "AnglePerLengthUom",
    "AnglePerVolumeMeasure",
    "AnglePerVolumeUom",
    "AngularAccelerationMeasure",
    "AngularAccelerationUom",
    "AngularVelocityMeasure",
    "AngularVelocityUom",
    "AreaMeasure",
    "AreaPerAmountOfSubstanceMeasure",
    "AreaPerAmountOfSubstanceUom",
    "AreaPerAreaMeasure",
    "AreaPerAreaUom",
    "AreaPerMassMeasure",
    "AreaPerMassUom",
    "AreaPerTimeMeasure",
    "AreaPerTimeUom",
    "AreaPerVolumeMeasure",
    "AreaPerVolumeUom",
    "AreaUom",
    "AttenuationPerFrequencyIntervalMeasure",
    "AttenuationPerFrequencyIntervalUom",
    "AxisOrder2d",
    "BinaryContactInterpretationPart",
    "BlockedWellboreRepresentation",
    "Boolean",
    "BooleanArrayFromDiscretePropertyArray",
    "BooleanArrayFromIndexArray",
    "BooleanConstantArray",
    "BooleanHdf5Array",
    "Boolean_PropertyType",
    "BoundaryFeature",
    "BoundaryFeatureInterpretation",
    "BoundaryRelation",
    "CI_Address",
    "CI_Address_PropertyType",
    "CI_Address_Type",
    "CI_Citation",
    "CI_Citation_PropertyType",
    "CI_Citation_Type",
    "CI_Contact",
    "CI_Contact_PropertyType",
    "CI_Contact_Type",
    "CI_Date",
    "CI_DateTypeCode",
    "CI_DateTypeCode_PropertyType",
    "CI_Date_PropertyType",
    "CI_Date_Type",
    "CI_OnLineFunctionCode",
    "CI_OnLineFunctionCode_PropertyType",
    "CI_OnlineResource",
    "CI_OnlineResource_PropertyType",
    "CI_OnlineResource_Type",
    "CI_PresentationFormCode",
    "CI_PresentationFormCode_PropertyType",
    "CI_ResponsibleParty",
    "CI_ResponsibleParty_PropertyType",
    "CI_ResponsibleParty_Type",
    "CI_RoleCode",
    "CI_RoleCode_PropertyType",
    "CI_Series",
    "CI_Series_PropertyType",
    "CI_Series_Type",
    "CI_Telephone",
    "CI_Telephone_PropertyType",
    "CI_Telephone_Type",
    "CRSPropertyType",
    "CapacitanceMeasure",
    "CapacitanceUom",
    "CartesianCSPropertyType",
    "CartesianCSType",
    "CartesianCS_1",
    "CategoricalProperty",
    "CategoricalPropertySeries",
    "CellFluidPhaseUnits",
    "CellOverlap",
    "CellParentWindow",
    "CellShape",
    "CellStratigraphicUnits",
    "CharacterString",
    "CharacterString_PropertyType",
    "ChronostratigraphicRank",
    "Citation",
    "CodeListValue_Type",
    "CodeType",
    "CodeWithAuthorityType",
    "ColumnLayerParentWindow",
    "ColumnLayerSplitColumnEdges",
    "ColumnLayerSplitCoordinateLines",
    "ColumnLayerSubnodeTopology",
    "ColumnShape",
    "ColumnSubnodePatch",
    "CommentProperty",
    "CommentPropertySeries",
    "ConnectionInterpretations",
    "ContactElementReference",
    "ContactIdentity",
    "ContactMode",
    "ContactPatch",
    "ContactRelationship",
    "ContactRepresentationReference",
    "ContactSide",
    "ContactVerb",
    "ContinuousProperty",
    "ContinuousPropertySeries",
    "CoordinateSystemAxis",
    "CoordinateSystemAxisPropertyType",
    "CoordinateSystemAxisType",
    "CustomData",
    "DQ_EvaluationMethodTypeCode",
    "DQ_EvaluationMethodTypeCode_PropertyType",
    "DQ_Result_PropertyType",
    "DataObjectParameter",
    "DataObjectReference",
    "DataTransferSpeedMeasure",
    "DataTransferSpeedUom",
    "Date",
    "DateTime",
    "DateTime_PropertyType",
    "Date_PropertyType",
    "Definition",
    "DefinitionBaseType",
    "DefinitionType",
    "DepositionMode",
    "DeviationSurveyRepresentation",
    "DiffusionCoefficientMeasure",
    "DiffusionCoefficientUom",
    "DigitalStorageMeasure",
    "DigitalStorageUom",
    "DimensionlessMeasure",
    "DimensionlessUom",
    "DipoleMomentMeasure",
    "DipoleMomentUom",
    "DiscreteProperty",
    "DiscretePropertySeries",
    "Domain",
    "DoseEquivalentMeasure",
    "DoseEquivalentUom",
    "DoubleConstantArray",
    "DoubleHdf5Array",
    "DoubleLatticeArray",
    "DoubleLookup",
    "DoubleTableLookup",
    "DynamicViscosityMeasure",
    "DynamicViscosityUom",
    "EX_Extent",
    "EX_Extent_Type",
    "EX_GeographicExtent_PropertyType",
    "EX_TemporalExtent",
    "EX_TemporalExtent_PropertyType",
    "EX_TemporalExtent_Type",
    "EX_VerticalExtent",
    "EX_VerticalExtent_PropertyType",
    "EX_VerticalExtent_Type",
    "EarthModelInterpretation",
    "EdgePatch",
    "Edges",
    "ElectricChargeMeasure",
    "ElectricChargePerAreaMeasure",
    "ElectricChargePerAreaUom",
    "ElectricChargePerMassMeasure",
    "ElectricChargePerMassUom",
    "ElectricChargePerVolumeMeasure",
    "ElectricChargePerVolumeUom",
    "ElectricChargeUom",
    "ElectricConductanceMeasure",
    "ElectricConductanceUom",
    "ElectricConductivityMeasure",
    "ElectricConductivityUom",
    "ElectricCurrentDensityMeasure",
    "ElectricCurrentDensityUom",
    "ElectricCurrentMeasure",
    "ElectricCurrentUom",
    "ElectricFieldStrengthMeasure",
    "ElectricFieldStrengthUom",
    "ElectricPotentialDifferenceMeasure",
    "ElectricPotentialDifferenceUom",
    "ElectricResistanceMeasure",
    "ElectricResistancePerLengthMeasure",
    "ElectricResistancePerLengthUom",
    "ElectricResistanceUom",
    "ElectricalResistivityMeasure",
    "ElectricalResistivityUom",
    "ElectromagneticMomentMeasure",
    "ElectromagneticMomentUom",
    "ElementIdentity",
    "ElementIndices",
    "EllipsoidPropertyType",
    "EllipsoidType",
    "Ellipsoid_1",
    "EllipsoidalCSPropertyType",
    "EllipsoidalCSType",
    "EllipsoidalCS_1",
    "EnergyLengthPerAreaMeasure",
    "EnergyLengthPerAreaUom",
    "EnergyLengthPerTimeAreaTemperatureMeasure",
    "EnergyLengthPerTimeAreaTemperatureUom",
    "EnergyMeasure",
    "EnergyPerAreaMeasure",
    "EnergyPerAreaUom",
    "EnergyPerLengthMeasure",
    "EnergyPerLengthUom",
    "EnergyPerMassMeasure",
    "EnergyPerMassPerTimeMeasure",
    "EnergyPerMassPerTimeUom",
    "EnergyPerMassUom",
    "EnergyPerVolumeMeasure",
    "EnergyPerVolumeUom",
    "EnergyUom",
    "EpcExternalPartReference",
    "Facet",
    "FaultInterpretation",
    "FaultThrow",
    "FloatingPointQuantityParameter",
    "FluidBoundaryFeature",
    "FluidContact",
    "FluidMarker",
    "ForceAreaMeasure",
    "ForceAreaUom",
    "ForceLengthPerLengthMeasure",
    "ForceLengthPerLengthUom",
    "ForceMeasure",
    "ForcePerForceMeasure",
    "ForcePerForceUom",
    "ForcePerLengthMeasure",
    "ForcePerLengthUom",
    "ForcePerVolumeMeasure",
    "ForcePerVolumeUom",
    "ForceUom",
    "FrequencyIntervalMeasure",
    "FrequencyIntervalUom",
    "FrequencyMeasure",
    "FrequencyUom",
    "FrontierFeature",
    "GeneralConversionPropertyType",
    "GenericFeatureInterpretation",
    "GeneticBoundaryFeature",
    "GeneticBoundaryKind",
    "Geobody3dShape",
    "GeobodyBoundaryInterpretation",
    "GeobodyFeature",
    "GeobodyInterpretation",
    "GeodeticCRS",
    "GeodeticCRSPropertyType",
    "GeodeticCRSType",
    "GeodeticDatumPropertyType",
    "GeodeticDatumType",
    "GeodeticDatum_1",
    "GeologicBoundaryKind",
    "GeologicUnitComposition",
    "GeologicUnitFeature",
    "GeologicUnitInterpretation",
    "GeologicUnitInterpretationIndex",
    "GeologicUnitMaterialImplacement",
    "GlobalChronostratigraphicColumn",
    "GmlProjectedCrsDefinition",
    "GmlVerticalCrsDefinition",
    "GpGridColumnLayerGrid",
    "GpGridIjkGridPatch",
    "GpGridRepresentation",
    "GpGridUnstructuredColumnLayerGridPatch",
    "GpGridUnstructuredGridPatch",
    "Grid2dPatch",
    "Grid2dRepresentation",
    "Grid2dSetRepresentation",
    "GridConnectionSetRepresentation",
    "GridGeometryAttachment",
    "Hdf5Dataset",
    "HeatCapacityMeasure",
    "HeatCapacityUom",
    "HeatFlowRateMeasure",
    "HeatFlowRateUom",
    "HeatTransferCoefficientMeasure",
    "HeatTransferCoefficientUom",
    "HorizonInterpretation",
    "HorizonInterpretationIndex",
    "HorizontalPlaneGeometry",
    "IdentifiedObjectType",
    "IdentityKind",
    "IjGaps",
    "IjSplitColumnEdges",
    "IjkGridGeometry",
    "IjkGridRepresentation",
    "IjkParentWindow",
    "IlluminanceMeasure",
    "IlluminanceUom",
    "IndexableElements",
    "InductanceMeasure",
    "InductanceUom",
    "IntegerArrayFromBooleanMaskArray",
    "IntegerConstantArray",
    "IntegerHdf5Array",
    "IntegerLatticeArray",
    "IntegerQuantityParameter",
    "IntegerRangeArray",
    "IntervalGridCells",
    "IntervalStratigraphicUnits",
    "Intervals",
    "IsothermalCompressibilityMeasure",
    "IsothermalCompressibilityUom",
    "KDirection",
    "KGaps",
    "KinematicViscosityMeasure",
    "KinematicViscosityUom",
    "LengthMeasure",
    "LengthPerLengthMeasure",
    "LengthPerLengthUom",
    "LengthPerMassMeasure",
    "LengthPerMassUom",
    "LengthPerPressureMeasure",
    "LengthPerPressureUom",
    "LengthPerTemperatureMeasure",
    "LengthPerTemperatureUom",
    "LengthPerTimeMeasure",
    "LengthPerTimeUom",
    "LengthPerVolumeMeasure",
    "LengthPerVolumeUom",
    "LengthUom",
    "LightExposureMeasure",
    "LightExposureUom",
    "LineRole",
    "LinearAccelerationMeasure",
    "LinearAccelerationUom",
    "LinearThermalExpansionMeasure",
    "LinearThermalExpansionUom",
    "LocalDepth3dCrs",
    "LocalGridSet",
    "LocalPropertyKind",
    "LocalTime3dCrs",
    "LogarithmicPowerRatioMeasure",
    "LogarithmicPowerRatioPerLengthMeasure",
    "LogarithmicPowerRatioPerLengthUom",
    "LogarithmicPowerRatioUom",
    "LuminanceMeasure",
    "LuminanceUom",
    "LuminousEfficacyMeasure",
    "LuminousEfficacyUom",
    "LuminousFluxMeasure",
    "LuminousFluxUom",
    "LuminousIntensityMeasure",
    "LuminousIntensityUom",
    "MD_Identifier",
    "MD_Identifier_PropertyType",
    "MD_Identifier_Type",
    "MagneticDipoleMomentMeasure",
    "MagneticDipoleMomentUom",
    "MagneticFieldStrengthMeasure",
    "MagneticFieldStrengthUom",
    "MagneticFluxDensityMeasure",
    "MagneticFluxDensityPerLengthMeasure",
    "MagneticFluxDensityPerLengthUom",
    "MagneticFluxDensityUom",
    "MagneticFluxMeasure",
    "MagneticFluxUom",
    "MagneticPermeabilityMeasure",
    "MagneticPermeabilityUom",
    "MagneticVectorPotentialMeasure",
    "MagneticVectorPotentialUom",
    "MassLengthMeasure",
    "MassLengthUom",
    "MassMeasure",
    "MassPerAreaMeasure",
    "MassPerAreaUom",
    "MassPerEnergyMeasure",
    "MassPerEnergyUom",
    "MassPerLengthMeasure",
    "MassPerLengthUom",
    "MassPerMassMeasure",
    "MassPerMassUom",
    "MassPerTimeMeasure",
    "MassPerTimePerAreaMeasure",
    "MassPerTimePerAreaUom",
    "MassPerTimePerLengthMeasure",
    "MassPerTimePerLengthUom",
    "MassPerTimeUom",
    "MassPerVolumeMeasure",
    "MassPerVolumePerLengthMeasure",
    "MassPerVolumePerLengthUom",
    "MassPerVolumeUom",
    "MassUom",
    "MdDatum",
    "MdDomain",
    "MdReference",
    "MobilityMeasure",
    "MobilityUom",
    "MolarEnergyMeasure",
    "MolarEnergyUom",
    "MolarHeatCapacityMeasure",
    "MolarHeatCapacityUom",
    "MolarVolumeMeasure",
    "MolarVolumeUom",
    "MolecularWeightMeasure",
    "MolecularWeightUom",
    "MomentOfForceMeasure",
    "MomentOfForceUom",
    "MomentOfInertiaMeasure",
    "MomentOfInertiaUom",
    "MomentumMeasure",
    "MomentumUom",
    "MultipleContactInterpretationPart",
    "NameValuePair",
    "NilReasonEnumeration_value",
    "NodePatch",
    "NodesPerCell",
    "NonSealedContactRepresentationPart",
    "NonSealedSurfaceFrameworkRepresentation",
    "NormalizedPowerMeasure",
    "NormalizedPowerUom",
    "ObjectAlias",
    "ObjectParameterKey",
    "OrderingCriteria",
    "OrganizationFeature",
    "OrganizationKind",
    "OrientedMacroFace",
    "OverlapVolume",
    "ParameterKind",
    "ParameterTemplate",
    "ParametricLineArray",
    "ParametricLineFromRepresentationGeometry",
    "ParametricLineFromRepresentationLatticeArray",
    "ParametricLineGeometry",
    "ParametricLineIntersections",
    "Patch",
    "Patch1d",
    "PatchBoundaries",
    "PatchOfGeometry",
    "PatchOfPoints",
    "PatchOfValues",
    "PermeabilityLengthMeasure",
    "PermeabilityLengthUom",
    "PermeabilityRockMeasure",
    "PermeabilityRockUom",
    "PermittivityMeasure",
    "PermittivityUom",
    "Phase",
    "PillarShape",
    "PlaneAngleMeasure",
    "PlaneAngleUom",
    "PlaneSetRepresentation",
    "Point2dHdf5Array",
    "Point3d",
    "Point3dFromRepresentationLatticeArray",
    "Point3dHdf5Array",
    "Point3dLatticeArray",
    "Point3dOffset",
    "Point3dParametricArray",
    "Point3dZValueArray",
    "PointGeometry",
    "PointSetRepresentation",
    "PointsProperty",
    "PolylineRepresentation",
    "PolylineSetPatch",
    "PolylineSetRepresentation",
    "PotentialDifferencePerPowerDropMeasure",
    "PotentialDifferencePerPowerDropUom",
    "PowerMeasure",
    "PowerPerAreaMeasure",
    "PowerPerAreaUom",
    "PowerPerPowerMeasure",
    "PowerPerPowerUom",
    "PowerPerVolumeMeasure",
    "PowerPerVolumeUom",
    "PowerUom",
    "PressureMeasure",
    "PressurePerTimeMeasure",
    "PressurePerTimeUom",
    "PressurePerVolumeMeasure",
    "PressurePerVolumeUom",
    "PressureSquaredMeasure",
    "PressureSquaredPerForceTimePerAreaMeasure",
    "PressureSquaredPerForceTimePerAreaUom",
    "PressureSquaredUom",
    "PressureTimePerVolumeMeasure",
    "PressureTimePerVolumeUom",
    "PressureUom",
    "PrimeMeridianPropertyType",
    "PrimeMeridianType",
    "PrimeMeridian_1",
    "ProjectedCRS",
    "ProjectedCRSType",
    "ProjectedCrsEpsgCode",
    "ProjectedCrs_1",
    "ProjectedUnknownCrs",
    "PropertyKind",
    "PropertyKindFacet",
    "PropertySet",
    "PropertyValuesPatch",
    "QuantityOfLightMeasure",
    "QuantityOfLightUom",
    "RadianceMeasure",
    "RadianceUom",
    "RadiantIntensityMeasure",
    "RadiantIntensityUom",
    "Real",
    "Real_PropertyType",
    "ReciprocalAreaMeasure",
    "ReciprocalAreaUom",
    "ReciprocalElectricPotentialDifferenceMeasure",
    "ReciprocalElectricPotentialDifferenceUom",
    "ReciprocalForceMeasure",
    "ReciprocalForceUom",
    "ReciprocalLengthMeasure",
    "ReciprocalLengthUom",
    "ReciprocalMassMeasure",
    "ReciprocalMassTimeMeasure",
    "ReciprocalMassTimeUom",
    "ReciprocalMassUom",
    "ReciprocalPressureMeasure",
    "ReciprocalPressureUom",
    "ReciprocalTimeMeasure",
    "ReciprocalTimeUom",
    "ReciprocalVolumeMeasure",
    "ReciprocalVolumeUom",
    "RedefinedGeometryRepresentation",
    "ReferenceType",
    "Regrid",
    "RelatedTimeType",
    "RelatedTimeType_relativePosition",
    "ReluctanceMeasure",
    "ReluctanceUom",
    "RepresentationIdentity",
    "RepresentationIdentitySet",
    "RepresentationSetRepresentation",
    "ResqmlJaggedArray",
    "ResqmlPropertyKind",
    "ResqmlUom",
    "RockFluidOrganizationInterpretation",
    "RockFluidUnitFeature",
    "RockFluidUnitInterpretation",
    "RockFluidUnitInterpretationIndex",
    "SC_CRS_PropertyType",
    "SealedContactRepresentationPart",
    "SealedSurfaceFrameworkRepresentation",
    "SealedVolumeFrameworkRepresentation",
    "SecondDefiningParameter_1",
    "SecondMomentOfAreaMeasure",
    "SecondMomentOfAreaUom",
    "Seismic2dCoordinates",
    "Seismic3dCoordinates",
    "SeismicLatticeFeature",
    "SeismicLatticeSetFeature",
    "SeismicLineFeature",
    "SeismicLineSetFeature",
    "SequenceStratigraphySurface",
    "SignalingEventPerTimeMeasure",
    "SignalingEventPerTimeUom",
    "SolidAngleMeasure",
    "SolidAngleUom",
    "SpecificHeatCapacityMeasure",
    "SpecificHeatCapacityUom",
    "SphericalCSPropertyType",
    "SphericalCSType",
    "SphericalCS_1",
    "SplitEdges",
    "SplitFaces",
    "SplitNodePatch",
    "StandardPropertyKind",
    "StratigraphicColumn",
    "StratigraphicColumnRankInterpretation",
    "StratigraphicOccurrenceInterpretation",
    "StratigraphicUnitFeature",
    "StratigraphicUnitInterpretation",
    "StratigraphicUnitInterpretationIndex",
    "StreamlineFlux",
    "StreamlinePolylineSetPatch",
    "StreamlineWellbores",
    "StreamlinesFeature",
    "StreamlinesRepresentation",
    "StringHdf5Array",
    "StringLookup",
    "StringOrRefType",
    "StringParameter",
    "StringTableLookup",
    "StructuralOrganizationInterpretation",
    "SubRepresentation",
    "SubRepresentationPatch",
    "SubnodeNodeObject",
    "SubnodePatch",
    "SubnodeTopology",
    "SurfaceRole",
    "TM_Primitive_PropertyType",
    "TectonicBoundaryFeature",
    "TectonicBoundaryKind",
    "TemperatureIntervalMeasure",
    "TemperatureIntervalPerLengthMeasure",
    "TemperatureIntervalPerLengthUom",
    "TemperatureIntervalPerPressureMeasure",
    "TemperatureIntervalPerPressureUom",
    "TemperatureIntervalPerTimeMeasure",
    "TemperatureIntervalPerTimeUom",
    "TemperatureIntervalUom",
    "ThermalConductanceMeasure",
    "ThermalConductanceUom",
    "ThermalConductivityMeasure",
    "ThermalConductivityUom",
    "ThermalDiffusivityMeasure",
    "ThermalDiffusivityUom",
    "ThermalInsulanceMeasure",
    "ThermalInsulanceUom",
    "ThermalResistanceMeasure",
    "ThermalResistanceUom",
    "ThermodynamicTemperatureMeasure",
    "ThermodynamicTemperatureUom",
    "ThreePoint3d",
    "ThrowKind",
    "TiltedPlaneGeometry",
    "TimeIndex",
    "TimeIndexParameter",
    "TimeIndexParameterKey",
    "TimeIndices",
    "TimeInterval",
    "TimeMeasure",
    "TimePerLengthMeasure",
    "TimePerLengthUom",
    "TimePerMassMeasure",
    "TimePerMassUom",
    "TimePerTimeMeasure",
    "TimePerTimeUom",
    "TimePerVolumeMeasure",
    "TimePerVolumeUom",
    "TimePrimitivePropertyType",
    "TimeSeries",
    "TimeSeriesParentage",
    "TimeSetKind",
    "TimeUom",
    "Timestamp",
    "TrianglePatch",
    "TriangulatedSetRepresentation",
    "TruncatedIjkGridRepresentation",
    "TruncatedUnstructuredColumnLayerGridRepresentation",
    "TruncationCellPatch",
    "URL",
    "URL_PropertyType",
    "UniformSubnodePatch",
    "UnstructuredColumnEdges",
    "UnstructuredColumnLayerGridGeometry",
    "UnstructuredColumnLayerGridRepresentation",
    "UnstructuredGridGeometry",
    "UnstructuredGridHingeNodeFaces",
    "UnstructuredGridRepresentation",
    "UnstructuredSubnodeTopology",
    "VariableSubnodePatch",
    "VerticalCRS",
    "VerticalCRSType",
    "VerticalCSPropertyType",
    "VerticalCSType",
    "VerticalCS_1",
    "VerticalCrsEpsgCode",
    "VerticalCrs_1",
    "VerticalDatumPropertyType",
    "VerticalDatumType",
    "VerticalDatum_1",
    "VerticalDirection",
    "VerticalUnknownCrs",
    "VolumeFlowRatePerVolumeFlowRateMeasure",
    "VolumeFlowRatePerVolumeFlowRateUom",
    "VolumeMeasure",
    "VolumePerAreaMeasure",
    "VolumePerAreaUom",
    "VolumePerLengthMeasure",
    "VolumePerLengthUom",
    "VolumePerMassMeasure",
    "VolumePerMassUom",
    "VolumePerPressureMeasure",
    "VolumePerPressureUom",
    "VolumePerRotationMeasure",
    "VolumePerRotationUom",
    "VolumePerTimeLengthMeasure",
    "VolumePerTimeLengthUom",
    "VolumePerTimeMeasure",
    "VolumePerTimePerAreaMeasure",
    "VolumePerTimePerAreaUom",
    "VolumePerTimePerLengthMeasure",
    "VolumePerTimePerLengthUom",
    "VolumePerTimePerPressureLengthMeasure",
    "VolumePerTimePerPressureLengthUom",
    "VolumePerTimePerPressureMeasure",
    "VolumePerTimePerPressureUom",
    "VolumePerTimePerTimeMeasure",
    "VolumePerTimePerTimeUom",
    "VolumePerTimePerVolumeMeasure",
    "VolumePerTimePerVolumeUom",
    "VolumePerTimeUom",
    "VolumePerVolumeMeasure",
    "VolumePerVolumeUom",
    "VolumeRegion",
    "VolumeShell",
    "VolumeUom",
    "VolumetricHeatTransferCoefficientMeasure",
    "VolumetricHeatTransferCoefficientUom",
    "VolumetricThermalExpansionMeasure",
    "VolumetricThermalExpansionUom",
    "WellboreFeature",
    "WellboreFrameRepresentation",
    "WellboreInterpretation",
    "WellboreMarker",
    "WellboreMarkerFrameRepresentation",
    "WellboreTrajectoryParentIntersection",
    "WellboreTrajectoryRepresentation",
    "WitsmlWellboreReference",
    "actuate_value",
    "anchorDefinition",
    "axis",
    "axisAbbrev",
    "axisDirection",
    "baseGeodeticCRS",
    "cartesianCS_2",
    "conversion",
    "coordinateOperationAccuracy",
    "description",
    "descriptionReference",
    "domainOfValidity",
    "ellipsoid_2",
    "ellipsoidalCS_2",
    "geodeticDatum_2",
    "greenwichLongitude",
    "identifier",
    "maximumValue",
    "minimumValue",
    "name",
    "obj_Activity",
    "obj_ActivityTemplate",
    "obj_BlockedWellboreRepresentation",
    "obj_BoundaryFeature",
    "obj_BoundaryFeatureInterpretation",
    "obj_CategoricalProperty",
    "obj_CategoricalPropertySeries",
    "obj_CommentProperty",
    "obj_CommentPropertySeries",
    "obj_ContinuousProperty",
    "obj_ContinuousPropertySeries",
    "obj_DeviationSurveyRepresentation",
    "obj_DiscreteProperty",
    "obj_DiscretePropertySeries",
    "obj_DoubleTableLookup",
    "obj_EarthModelInterpretation",
    "obj_EpcExternalPartReference",
    "obj_FaultInterpretation",
    "obj_FluidBoundaryFeature",
    "obj_FrontierFeature",
    "obj_GenericFeatureInterpretation",
    "obj_GeneticBoundaryFeature",
    "obj_GeobodyBoundaryInterpretation",
    "obj_GeobodyFeature",
    "obj_GeobodyInterpretation",
    "obj_GeologicUnitFeature",
    "obj_GeologicUnitInterpretation",
    "obj_GlobalChronostratigraphicColumn",
    "obj_GpGridRepresentation",
    "obj_Grid2dRepresentation",
    "obj_Grid2dSetRepresentation",
    "obj_GridConnectionSetRepresentation",
    "obj_HorizonInterpretation",
    "obj_IjkGridRepresentation",
    "obj_LocalDepth3dCrs",
    "obj_LocalGridSet",
    "obj_LocalTime3dCrs",
    "obj_MdDatum",
    "obj_NonSealedSurfaceFrameworkRepresentation",
    "obj_OrganizationFeature",
    "obj_PlaneSetRepresentation",
    "obj_PointSetRepresentation",
    "obj_PointsProperty",
    "obj_PolylineRepresentation",
    "obj_PolylineSetRepresentation",
    "obj_PropertyKind",
    "obj_PropertySet",
    "obj_RedefinedGeometryRepresentation",
    "obj_RepresentationIdentitySet",
    "obj_RepresentationSetRepresentation",
    "obj_RockFluidOrganizationInterpretation",
    "obj_RockFluidUnitFeature",
    "obj_RockFluidUnitInterpretation",
    "obj_SealedSurfaceFrameworkRepresentation",
    "obj_SealedVolumeFrameworkRepresentation",
    "obj_SeismicLatticeFeature",
    "obj_SeismicLineFeature",
    "obj_SeismicLineSetFeature",
    "obj_StratigraphicColumn",
    "obj_StratigraphicColumnRankInterpretation",
    "obj_StratigraphicOccurrenceInterpretation",
    "obj_StratigraphicUnitFeature",
    "obj_StratigraphicUnitInterpretation",
    "obj_StreamlinesFeature",
    "obj_StreamlinesRepresentation",
    "obj_StringTableLookup",
    "obj_StructuralOrganizationInterpretation",
    "obj_SubRepresentation",
    "obj_TectonicBoundaryFeature",
    "obj_TimeSeries",
    "obj_TriangulatedSetRepresentation",
    "obj_TruncatedIjkGridRepresentation",
    "obj_TruncatedUnstructuredColumnLayerGridRepresentation",
    "obj_UnstructuredColumnLayerGridRepresentation",
    "obj_UnstructuredGridRepresentation",
    "obj_WellboreFeature",
    "obj_WellboreFrameRepresentation",
    "obj_WellboreInterpretation",
    "obj_WellboreMarkerFrameRepresentation",
    "obj_WellboreTrajectoryRepresentation",
    "operationVersion",
    "primeMeridian_2",
    "rangeMeaning",
    "realizationEpoch",
    "remarks",
    "scope",
    "secondDefiningParameter_2",
    "semiMajorAxis",
    "show_value",
    "sourceCRS",
    "sphericalCS_2",
    "targetCRS",
    "verticalCS_2",
    "verticalDatum_2",
]
